@@ -1,13 +1,11 @@
 ################################################################################
-### Part of the surveillance package, http://surveillance.r-forge.r-project.org
-### Free software under the terms of the GNU General Public License, version 2,
+### Compute one-step-ahead predictions at a series of time points
+###
+### Copyright (C) 2011-2012 Michaela Paul, 2012-2018 Sebastian Meyer
+###
+### This file is part of the R package "surveillance",
+### free software under the terms of the GNU General Public License, version 2,
 ### a copy of which is available at http://www.r-project.org/Licenses/.
-###
-### Compute one-step-ahead predictions (means) at a series of time points
-###
-### Copyright (C) 2011-2012 Michaela Paul, 2012-2015 Sebastian Meyer
-### $Revision: 1508 $
-### $Date: 2015-11-04 15:44:19 +0100 (Mit, 04. Nov 2015) $
 ################################################################################
 
 
@@ -22,7 +20,7 @@ oneStepAhead <- function(result, # hhh4-object (i.e. a hhh4 model fit)
                          cores = 1) # if which.start="final", the predictions
                                     # can be computed in parallel
 {
-    stopifnot(inherits(result, c("ah4", "hhh4")))
+    stopifnot(inherits(result, "hhh4"))
     type <- match.arg(type)
     if (type == "rolling" && !is.list(which.start)) {
         ## new in surveillance 1.10-0: if 'which.start' is a list, it is
@@ -36,19 +34,26 @@ oneStepAhead <- function(result, # hhh4-object (i.e. a hhh4 model fit)
     ## get model terms
     model <- result[["terms"]]
     if (is.null(model))
-        model <- result$terms <- with(result, interpretControl(control, stsObj))
-    nTime <- model$nTime
-    nUnits <- model$nUnits
+        model <- result$terms <- terms(result)
+    nTime <- model$nTime   # = nrow(result$stsObj)
+    nUnits <- model$nUnits # = ncol(result$stsObj)
     dimPsi <- model$nOverdisp
     withPsi <- dimPsi > 0L
     psiIdx <- model$nFE + model$nd + seq_len(dimPsi)
-    
+
     ## check that tp is within the time period of the data
-    maxlag <- if (is.null(result$lags) || all(is.na(result$lags)))
-        1L else max(result$lags, na.rm=TRUE)
-    stopifnot(tp %in% seq.int(maxlag,nTime-1L), length(tp) %in% 1:2)
-    if (length(tp) == 1) tp <- c(tp, max(model$subset)-1)
-    tps <- tp[1]:tp[2]
+    stopifnot(length(tp) %in% 1:2, tp >= 0)
+    tpRange <- c(model$subset[1L], nTime-1L) # supported range
+    if (any(tp > tpRange[2L]) || (type != "final" && any(tp < tpRange[1L]))) {
+        stop("the time range defined by 'tp' must be a subset of ",
+             tpRange[1L], ":", tpRange[2L])
+    }
+    if (length(tp) == 1) {
+        tp <- c(tp, max(model$subset)-1L)  # historical default
+        if (tp[1L] > tp[2L])  # probably unintended
+            stop("'tp' larger than the default upper limit (", tp[2L], ")")
+    }
+    tps <- tp[1L]:tp[2L]  # this function actually works if tp[1] > tp[2]
     ntps <- length(tps)
     observed <- model$response[tps+1,,drop=FALSE]
     rownames(observed) <- tps+1
@@ -58,7 +63,7 @@ oneStepAhead <- function(result, # hhh4-object (i.e. a hhh4 model fit)
     result$control$verbose <- max(0, verbose - (ntps>1))
     if (type != "rolling" && verbose > 1L) verbose <- 1L
     do_pb <- verbose == 1L && interactive()
-    
+
     ## initial fit
     fit <- if (type == "first") {
         if (do_pb)
@@ -132,7 +137,7 @@ oneStepAhead <- function(result, # hhh4-object (i.e. a hhh4 model fit)
             Sigma.orig[] <- .extractFromList("Sigma.orig")
             logliks[] <- .extractFromList("logliks")
         }
-        
+
     } else { ## sequential one-step ahead predictions
 
         if (do_pb) pb <- txtProgressBar(min=0, max=ntps, initial=0, style=3)
@@ -140,7 +145,7 @@ oneStepAhead <- function(result, # hhh4-object (i.e. a hhh4 model fit)
             if (verbose > 1L) {
                 cat("\nOne-step-ahead prediction @ t =", tps[i], "...\n")
             } else if (do_pb) setTxtProgressBar(pb, i)
-            
+
             if (type == "rolling") { # update fit
                 fit.old <- fit # backup
                 start <- if (is.list(which.start)) {
@@ -161,7 +166,7 @@ oneStepAhead <- function(result, # hhh4-object (i.e. a hhh4 model fit)
             }
 
             res <- getPreds(fit, tps[i])
-            
+
             ## gather results
             pred[i,] <- res$pred
             if (withPsi)
@@ -178,9 +183,9 @@ oneStepAhead <- function(result, # hhh4-object (i.e. a hhh4 model fit)
 
     ## with shared overdispersion parameters we need to expand psi to ncol(pred)
     if (dimPsi > 1L && dimPsi != nUnits) {
-        psi <- psi[,model$indexPsi]
+        psi <- psi[,model$indexPsi,drop=FALSE]
     }
-    
+
     ## done
     res <- c(list(pred = pred, observed = observed,
            psi = if (withPsi) psi else NULL,
@@ -208,7 +213,61 @@ psi2size.oneStepAhead <- function (object)
         size <- rep.int(size, dimpred[2L])
         dim(size) <- dimpred
     }
-    
+
     dimnames(size) <- list(rownames(object$psi), colnames(object$pred))
     size
 }
+
+## quantiles of the one-step-ahead forecasts
+quantile.oneStepAhead <- function (x, probs = c(2.5, 10, 50, 90, 97.5)/100, ...)
+{
+    stopifnot(is.vector(probs, mode = "numeric"), probs >= 0, probs <= 1,
+              (np <- length(probs)) > 0)
+    names(probs) <- paste(format(100*probs, trim=TRUE, scientific=FALSE, digits=3), "%")
+
+    size <- psi2size.oneStepAhead(x)
+    qs <- if (is.null(size)) {
+        vapply(X = probs, FUN = qpois, FUN.VALUE = x$pred,
+               lambda = x$pred)
+    } else {
+        vapply(X = probs, FUN = qnbinom, FUN.VALUE = x$pred,
+               mu = x$pred, size = size)
+    }
+    ## one tp, one unit -> qs is a vector of length np
+    ## otherwise, 'qs' has dimensions ntps x nUnit x np
+    ## if nUnit==1, we return an ntps x np matrix, otherwise an array
+    if (is.vector(qs)) {
+        qs <- t(qs)
+        rownames(qs) <- rownames(x$pred)
+        qs
+    } else if (dim(qs)[2L] == 1L) {
+        matrix(qs, dim(qs)[1L], dim(qs)[3L], dimnames = dimnames(qs)[c(1L,3L)])
+    } else qs
+}
+
+## confidence intervals for one-step-ahead predictions
+confint.oneStepAhead <- function (object, parm, level = 0.95, ...)
+{
+    quantile.oneStepAhead(object, (1+c(-1,1)*level)/2, ...)
+}
+
+## simple plot of one-step-ahead forecasts
+plot.oneStepAhead <- function (x, unit = 1, probs = 1:99/100,
+                               start = NULL, means.args = NULL, ...)
+{
+    stopifnot(length(unit) == 1, length(probs) > 1)
+
+    ## select unit
+    obs <- x$observed[,unit]
+    ms <- x$pred[,unit]
+    qs <- quantile.oneStepAhead(x, probs = probs)
+    if (!is.matrix(qs))  # multi-unit predictions
+        qs <- matrix(qs[,unit,], dim(qs)[1L], dim(qs)[3L],
+                     dimnames = dimnames(qs)[c(1L,3L)])
+
+    ## produce fanplot
+    if (is.null(start))
+        start <- as.integer(rownames(qs)[1L])
+    fanplot(quantiles = qs, probs = probs, means = ms,
+            observed = obs, start = start, means.args = means.args, ...)
+}
