@@ -6,9 +6,9 @@
 ### Maximum Likelihood inference for the two-component spatio-temporal intensity
 ### model described in Meyer et al (2012), DOI: 10.1111/j.1541-0420.2011.01684.x
 ###
-### Copyright (C) 2009-2017 Sebastian Meyer
-### $Revision: 1983 $
-### $Date: 2017-10-06 09:04:54 +0200 (Fri, 06. Oct 2017) $
+### Copyright (C) 2009-2018 Sebastian Meyer
+### $Revision: 2117 $
+### $Date: 2018-04-19 09:08:09 +0200 (Thu, 19. Apr 2018) $
 ################################################################################
 
 
@@ -80,7 +80,7 @@ twinstim <- function (
         if (is.na(justBeforet0)) justBeforet0 <- length(data$stgrid$start)   # t0 was too big
         if (justBeforet0 == 0L) justBeforet0 <- 1L   # t0 was too small
         t0 <- data$stgrid$start[justBeforet0]
-        message("replaced 't0' by the value ", t0,
+        warning("replaced 't0' by the value ", t0,
                 " (must be a 'start' time of 'data$stgrid')")
     }
     if (!T %in% data$stgrid$stop) {
@@ -88,7 +88,7 @@ twinstim <- function (
         # if 'T' is beyond the time range covered by 'data$stgrid'
         if (is.na(justAfterT)) justAfterT <- length(data$stgrid$stop)   # T was too big
         T <- data$stgrid$stop[justAfterT]
-        message("replaced 'T' by the value ", T,
+        warning("replaced 'T' by the value ", T,
                 " (must be a 'stop' time of 'data$stgrid')")
     }
 
@@ -335,7 +335,7 @@ twinstim <- function (
         if (nbeta0 == 1L) { # global intercept
             hEventsExpr <- call("+", quote(beta0), hEventsExpr)
         } else if (nbeta0 > 1L) { # type-specific intercept
-            hEventsExpr <- call("+", quote(beta0[eventTypes]), hEventsExpr)
+            hEventsExpr <- call("+", quote(beta0[eventTypes[includes]]), hEventsExpr)
         }
         if (!is.null(offsetEvents))
             hEventsExpr <- call("+", quote(offsetEvents), hEventsExpr)
@@ -687,12 +687,12 @@ twinstim <- function (
                 sEventsSum - unname(sInt)
             }) else if (nbeta0 > 1L) local({ # type-specific intercepts
                 ind <- sapply(seq_len(nTypes),
-                              function (type) eventTypes == type,
-                              simplify=TRUE, USE.NAMES=FALSE) # logical N x nTypes matrix
+                              function (type) eventTypes[includes] == type,
+                              simplify=TRUE, USE.NAMES=FALSE) # logical Nin x nTypes matrix
                 sEvents <- if (hase) {
                         ind * hEvents / lambdaEvents
                     } else ind
-                sEventsSum <- .colSums(sEvents, N, nTypes)
+                sEventsSum <- .colSums(sEvents, Nin, nTypes)
                 sInt <- exp(beta0) * .hIntTW(beta)
                 sEventsSum - unname(sInt)
             }) else numeric(0L) # i.e. nbeta0 == 0L
@@ -789,8 +789,8 @@ twinstim <- function (
         hScoreEvents <- if (hash) {
             scoreEvents_beta0 <- if (nbeta0 > 1L) local({ # type-specific intercepts
                 ind <- sapply(seq_len(nTypes),
-                              function (type) eventTypes == type,
-                              simplify=TRUE, USE.NAMES=FALSE) # logical N x nTypes matrix
+                              function (type) eventTypes[includes] == type,
+                              simplify=TRUE, USE.NAMES=FALSE) # logical Nin x nTypes matrix
                 if (hase) {
                     ind * hEvents / lambdaEvents
                 } else ind
