@@ -7,9 +7,9 @@
 ### class "twinstim". The function basically uses Ogata's modified thinning
 ### algorithm (cf. Daley & Vere-Jones, 2003, Algorithm 7.5.V.).
 ###
-### Copyright (C) 2010-2017 Sebastian Meyer
-### $Revision: 1993 $
-### $Date: 2017-10-06 15:25:39 +0200 (Fri, 06. Oct 2017) $
+### Copyright (C) 2010-2018 Sebastian Meyer
+### $Revision: 2178 $
+### $Date: 2018-07-17 11:04:22 +0200 (Tue, 17. Jul 2018) $
 ################################################################################
 
 ### CAVE:
@@ -91,6 +91,10 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
     	unionSpatialPolygons(tiles)
     } else check_W(W)  # does as(W, "SpatialPolygons")
 
+    if (!.skipChecks) {
+        cat("Checking 'tiles' ...\n")
+        ## we always check 'tiles', but quietly in the simulate-method
+    }
     tileLevels <- levels(stgrid$tile)
     tiles <- check_tiles(tiles, tileLevels,
                          areas.stgrid = stgrid[["area"]][seq_along(tileLevels)],
@@ -98,7 +102,6 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
 
     ## Transform W to class "owin"
     Wowin <- as(W, "owin")
-    Wedges <- edges(Wowin, check = FALSE)
     maxExtentOfW <- diameter.owin(Wowin)
 
 
@@ -198,6 +201,8 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
     if (Nout > 0L) {
         check_tiles_events(tiles, events)
         eventCoords <- coordinates(events)
+        rownames(eventCoords) <- NULL  # to avoid duplicates ("" for new events)
+                                       # which disturb the final SpatialPointsDataFrame()
         eventData <- events@data
         ## check presence of unpredictable marks
         if (length(.idx <- which(!unpredMarks %in% names(eventData)))) {
@@ -287,7 +292,7 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
             # actually, the process might be endemic offset-only, which I don't care about ATM
         }
         if (Nout == 0L) {
-            stop("missing 'events' pre-history (no endemic component)")
+            stop("missing 'events' prehistory (no endemic component)")
         }
     }
 
@@ -325,7 +330,7 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
         structure(mmhEvents, offset = model.offset(mfhEvents))
     }
 
-    # actually, we don't need the endemic model matrix for the pre-history events at all
+    # actually, we don't need the endemic model matrix for the prehistory events at all
     # this is just to test consistence with 'beta' and for the names of 'beta'
     mmh <- buildmmh(eventData[0L,])
     if (ncol(mmh) != p) {
@@ -455,7 +460,7 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
         eventTypes <- as.integer(eventData$type)
         eps.s <- eventData$eps.s
         # distance to the border (required for siafInt below, and for epidataCS)
-        bdist <- bdist(eventCoords, Wedges)
+        bdist <- bdist(eventCoords, Wowin)
         # spatial influence regions of the events
         influenceRegion <- if (nrow(eventCoords) > 0L) .influenceRegions(
             events = SpatialPointsDataFrame(
